/*========================================================================================================
 * AnalogInput.cpp
 *========================================================================================================
 *
 * File Description:
 *
 * This class extends the RunMotorsDrive class that runs the drive motors using an analog
 * input to control the speed/direction (forward/backward) of the robot.  The midpoint of
 * the analog range will yield no motor speed.  Values above this value will move the robot
 * forward and values below this value will move the robot backward.  Thus, it will convert
 * the voltage range to a -1.0 to 1.0 motor speed.
 *========================================================================================================
 *  Modifications:
 *  Date            Name            Description of Change
 *  18-Oct-2016     Joe Witcpalek   Created File
 *========================================================================================================*/

//  WPILIB includes
#include <AnalogInput.h>
#include <SmartDashboard/SmartDashboard.h>

// Team 302 includes
#include <AnalogInputDrive.h>           // This class
#include <RunMotorsDrive.h>             // Base class
#include <RobotMap.h>                   // Definition of ports



//--------------------------------------------------------------------
// Method:      AnalogInputDrive  <<constructor>>
// Description: Create and initialize object
//--------------------------------------------------------------------
AnalogInputDrive::AnalogInputDrive() : RunMotorsDrive(),
                                       m_sensor( nullptr )
{
    m_sensor = new AnalogInput( ANALOG_SENSOR_PORT );
    init();
}

//--------------------------------------------------------------------
// Method:      Init
// Description: This method will initialize the values.
// Returns:     void
//--------------------------------------------------------------------
void AnalogInputDrive::Init()
{
    SetCoastMode();
}


//--------------------------------------------------------------------
// Method:      HasTarget
// Description: This indicates if this drive mode doesn't have a target
// Returns:     false = doesn't have a target
//--------------------------------------------------------------------
bool AnalogInputDrive::HasTarget() const
{
    return false;
}


//--------------------------------------------------------------------
// Method:      SetTarget
// Description: This sets the target for the drive class.
// Returns:     void
//--------------------------------------------------------------------
void AnalogInputDrive::SetTarget
(
    float   target
)
{
}

//--------------------------------------------------------------------
// Method:      DrivePeriodic
// Description: This method will be called during the periodic loop
//              to drive the robot.  It will interact with the analog
//              input to control the speed/direction (forward/reverse)
//              to drive the robot.
// Returns:     false = this mode is never done
//--------------------------------------------------------------------
bool AnalogInputDrive::DrivePeriodic() const
{
    // TODO:    Convert this value to a motor input value between -1.0 and
    //          1.0 where:
    //              - 2.5 volts yields no movement.
    //                0.2 volts yields -1.0
    //                4.8 volts yields 1.0
    //          Update the motor speed using a base class method.

    // Read the raw voltage (between 0.2 and 4.8 volts)
    float voltage = m_sensor->GetVoltage();

    // Convert the voltage to the motor input value
    //
    // This is done by taking the voltage and subtracting the middle voltage value
    // then dividing the result by half the range.
    //
    // There is no need to limit the range because the ChangeSpeed call will do this
    // However, remove values between -0.1 and 0.1, so that these become 0.0.
    // This is called a deadband.

    float motorInput = 0.0;

    // Update the motor speeds
    ChangeSpeed( motorInput );

    // Indicate that we are not done
    return false;
}

//--------------------------------------------------------------------
// Method:      ~AnalogInputDrive  <<destructor>>
// Description: Clean up memory.
//--------------------------------------------------------------------
AnalogInputDrive::~AnalogInputDrive()
{
    delete m_sensor;
    m_sensor = nullptr;
}


//--------------------------------------------------------------------
// Method:      GetIdentifier
// Description: This gets the name that identifies the drive mode..
// Returns:     const char*         identifier for the drive
//--------------------------------------------------------------------
const char* AnalogInputDrive::GetIdentifier() const
{
    return "Analog Input Drive";
}
