/*========================================================================================================
 * AnalogInput.h
 *========================================================================================================
 *
 * File Description:
 *
 * This class extends the RunMotorsDrive class that runs the drive motors using an analog
 * input to control the speed/direction (forward/backward) of the robot.  The midpoint of
 * the analog range will yield no motor speed.  Values above this value will move the robot
 * forward and values below this value will move the robot backward.  Thus, it will convert
 * the voltage range to a -1.0 to 1.0 motor speed.
 *========================================================================================================
 *  Modifications:
 *  Date            Name            Description of Change
 *  18-Oct-2016     Joe Witcpalek   Created File
 *========================================================================================================*/

#ifndef SRC_ANALOGINPUTDRIVE_H_
#define SRC_ANALOGINPUTDRIVE_H_

//  WPILIB includes
#include <AnalogInput.h>

//  Team 302 includes
#include <RunMotorsDrive.h>             // Base class

class AnalogInputDrive: public RunMotorsDrive
{
    public:
        //--------------------------------------------------------------------
        // Method:      AnalogInputDrive  <<constructor>>
        // Description: Create and initialize object
        //--------------------------------------------------------------------
        AnalogInputDrive(); // constructor

        //--------------------------------------------------------------------
        // Method:      Init
        // Description: This method will initialize the values.
        // Returns:     void
        //--------------------------------------------------------------------
        void Init() override;

        //--------------------------------------------------------------------
        // Method:      HasTarget
        // Description: This indicates if this drive mode has a target
        //              such as time, distance, etc.
        // Returns:     true  = has a target
        //              false = doesn't have a target
        //--------------------------------------------------------------------
        bool HasTarget() const override;


        //--------------------------------------------------------------------
        // Method:      SetTarget
        // Description: This sets the target for the drive class.
        // Returns:     void
        //--------------------------------------------------------------------
        void SetTarget
        (
            float   target
        ) override;

        //--------------------------------------------------------------------
        // Method:      DrivePeriodic
        // Description: This method will be called during the periodic loop
        //              to drive the robot.  It will interact with the analog
        //              input to control the speed/direction (forward/reverse)
        //              to drive the robot.
        // Returns:     false = this mode is never done
        //--------------------------------------------------------------------
        bool DrivePeriodic() const override;

        //--------------------------------------------------------------------
        // Method:      GetIdentifier
        // Description: This gets the name that identifies the drive mode..
        // Returns:     const char*         identifier for the drive
        //--------------------------------------------------------------------
        const char* GetIdentifier() const override;

        //--------------------------------------------------------------------
        // Method:      ~AnalogInputDrive  <<destructor>>
        // Description: Clean up memory.
        //--------------------------------------------------------------------
        virtual ~AnalogInputDrive();    // destructor

    protected:

    private:
        const char*     m_SENSOR_NAME = "Analog Sensor";
        const char*     m_SENSOR_VOLTAGE = "Analog Sensor Voltage";
        const char*     m_SENSOR_VALUE = "Analog Sensor Value";
        const float     m_MIDDLE_VOLTAGE_VALUE = 2.5;
        const float     m_HALF_VOLTAGE_RANGE = 2.3; // 4.8 volts - 0.2 volts yields 4.6 volts, so half
                                                // the range (half positive and half negative) is 2.3
        AnalogInput*    m_sensor;
};

#endif /* SRC_ANALOGINPUTDRIVE_H_ */
