
 /*========================================================================================================
 * Robot.cpp
 *========================================================================================================
 *
 * File Description:
 *
 * This file defines the main robot class
 *========================================================================================================
 *  Modifications:
 *  Date            Name            Description of Change
 *  18-Oct-2016     Joe Witcpalek   Created File
 *========================================================================================================*/

// WPILib includes
#include <IterativeRobot.h>                 // Base Class

#include <SmartDashboard/SendableChooser.h> // Used to select drive mode
#include <SmartDashboard/SmartDashboard.h>  // Smart Dashboard



// Team 302 includes
#include <IDrive.h>
#include <IDriveFactory.h>
#include <AnalogInputDrive.h>
#include <ArcadeDrive.h>
#include <DigitalInputDrive.h>
#include <DriveToDistance.h>
#include <NoMoveDrive.h>
#include <OperatorInterface.h>
#include <RunMotorsDrive.h>
#include <TankDrive.h>
#include <TimedDrive.h>



class Robot: public IterativeRobot
{
    private:
        IDriveFactory*      m_driveFactory;     // drive factory to create drive modes
        IDrive*             m_drive;            // current drive class
        IDRIVE_IDENTIFIER   m_currentDriveMode; // current drive mode
        OperatorInterface*  m_oi;               // controller interface

        const std::string   m_driveMode = "Drive Mode:  ";


        // Constants for the auton chooser
        const std::string   m_driveModeNothing = "Do Nothing";
        const std::string   m_driveModeTimed = "Timed Drive";
        const std::string   m_driveModeDistance = "Drive Distance";

        // Objects used by auton
        SendableChooser*    m_autonChooser;         // Selections for auto on the dashboard
        std::string         m_selectedAutonMode;    // selected option on the m_autoChooser
        IDrive*             m_autonDrive;           // drive mode to run in auton


    public:

        //----------------------------------------------------------------------------------
        // Method:      Robot       <<constructor>>
        // Description: Create and intialize the object.
        //----------------------------------------------------------------------------------
        Robot() : IterativeRobot(),
                  m_driveFactory( nullptr ),
                  m_drive( nullptr ),
                  m_currentDriveMode( TANK_DRIVE ),
                  m_oi( nullptr ),
                  m_autonChooser( nullptr ),
                  m_autonDrive( nullptr )

        {
            m_driveFactory = IDriveFactory::GetInstance();
            m_drive = m_driveFactory->GetIDrive( TANK_DRIVE, 0.0 );
            m_oi = OperatorInterface::GetInstance();        // Cause the object to get created/initialized
        }

        //----------------------------------------------------------------------------------
        // Method:      RobotInit
        // Description: Initialization routine that is performed once upon the when
        //              the robot is initialized.  This happens before the robot does
        //              anything.
        // Returns:     void
        //----------------------------------------------------------------------------------
        void RobotInit()
        {
            // TODO:  Create a sendable chooser and add the auton items to the chooser
            //        put it on the dashboard
            // Put the auton choices on the dashboard
            /*
            m_autonChooser = new SendableChooser();
            m_autonChooser->AddDefault( m_driveModeNothing, (void*)&m_driveModeNothing );
            m_autonChooser->AddDefault( m_driveModeRunMotors, (void*)&m_driveModeRunMotors );
            m_autonChooser->AddObject(m_driveModeTimed, (void*)&m_driveModeTimed );
            m_autonChooser->AddObject(m_driveModeDistance, (void*)&m_driveModeDistance );
            SmartDashboard::PutData("Auton Modes", m_autonChooser );
            */

        }

        //----------------------------------------------------------------------------------
        // Method:      DisabledInit
        // Description: Initialization routine that gets called when the robot enters
        //              the disabled mode.
        // Returns:     void
        //----------------------------------------------------------------------------------
        void DisabledInit()
        {
        }

        //----------------------------------------------------------------------------------
        // Method:      DisabledPeriodic
        // Description: This runs after the disabled mode initialization has completed.
        //              It runs every loop (approximately every 20 milliseconds)  while
        //              the robot is disabled.
        // Returns:     void
        //----------------------------------------------------------------------------------
        void DisabledPeriodic()
        {
        }

        //----------------------------------------------------------------------------------
        // Method:      AutonomousInit
        // Description: Initialization routine that gets called when the robot enters
        //              the autonomous (no human interation) mode.
        // Returns:     void
        //----------------------------------------------------------------------------------
        void AutonomousInit()
        {
            // TODO:    After all of the pieces have been constructed and tested,
            //          connect the commands as follows:
            //          1) Run motors for 2 seconds and stop the motors
            //          2) Drive forward for 1.5 feet and stop the motors
            //          3) Drive backward 1.5 feet and stop the motors
            //          Did you end up at the same place where you ended the timed run?
            //          Why not?  What are some things that can be done to make it closer?
            // m_autonDrive = nullptr;
            // m_selectedAutonMode = *((std::string*) m_autonChooser->GetSelected() );
        }

        //----------------------------------------------------------------------------------
        // Method:      AutonomousPeriodic
        // Description: This runs after the autonomous mode initialization has completed.
        //              It runs every loop (approximately every 20 milliseconds)  while
        //              the robot is in autonomous mode.
        // Returns:     void
        //----------------------------------------------------------------------------------
        void AutonomousPeriodic()
        {
            // m_autonDrive->DrivePeriodic();
        }

        //----------------------------------------------------------------------------------
        // Method:      TeleopInit
        // Description: Initialization routine that gets called when the robot enters
        //              the teleop (human controlled) mode.
        // Returns:     void
        //----------------------------------------------------------------------------------
        void TeleopInit()
        {
            SmartDashboard::PutData( m_driveMode, m_drive->GetIdentifier() );
        }

        //----------------------------------------------------------------------------------
        // Method:      TeleopPeriodic
        // Description: This runs after the teleop mode initialization has completed.
        //              It runs every loop (approximately every 20 milliseconds)  while
        //              the robot is in teleop mode.
        // Returns:     void
        //----------------------------------------------------------------------------------
        void TeleopPeriodic()
        {
            BUTTON_STATE state = m_oi->GetButtonState( SWITCH_DRIVE_MODE );
            if ( state == BUTTON_PRESSED )
            {
                float target = 0.0;
                switch ( m_currentDriveMode )
                {
                case TANK_DRIVE:
                    m_currentDriveMode = ARCADE_DRIVE;
                    break;

                case ARCADE_DRIVE:
                    m_currentDriveMode = ARCADE_1STICK_DRIVE;
                    break;

                case ARCADE_1STICK_DRIVE:
                    m_currentDriveMode = TRIGGER_DRIVE;
                    break;

                case TRIGGER_DRIVE:
                    m_currentDriveMode = DIGITAL_INPUT_DRIVE;
                    break;

                case DIGITAL_INPUT_DRIVE:
                    m_currentDriveMode = ANALOG_INPUT_DRIVE;
                    break;

                case ANALOG_INPUT_DRIVE:
                    m_currentDriveMode = DRIVE_TO_DISTANCE;
                    target = 5.0;
                    break;

                case DRIVE_TO_DISTANCE:
                    m_currentDriveMode = RUN_MOTORS_DRIVE;
                    break;

                case RUN_MOTORS_DRIVE:
                    m_currentDriveMode = TIMED_DRIVE;
                    target = 3.0;
                    break;

                case TIMED_DRIVE:
                    m_currentDriveMode = NO_MOVE_DRIVE;
                    break;

                case NO_MOVE_DRIVE:
                    m_currentDriveMode = TANK_DRIVE;
                    break;

                default:
                    m_currentDriveMode = TANK_DRIVE;
                    break;
                }
                m_drive = m_driveFactory->GetIDrive( m_currentDriveMode, target );
                SmartDashboard::PutData( m_driveMode, m_drive->GetIdentifier() );
            }
            m_drive->DrivePeriodic();
        }

        //----------------------------------------------------------------------------------
        // Method:      TestInit
        // Description: Initialization routine that gets called when the robot enters
        //              the test (diagnostics) mode.
        // Returns:     void
        //----------------------------------------------------------------------------------
        void TestInit()
        {
        }

        //----------------------------------------------------------------------------------
        // Method:      TestPeriodic
        // Description: This runs after the test mode initialization has completed.
        //              It runs every loop (approximately every 20 milliseconds)  while
        //              the robot is in test mode.
        // Returns:     void
        //----------------------------------------------------------------------------------
        void TestPeriodic()
        {
        }
};

START_ROBOT_CLASS(Robot)
