/*========================================================================================================
 * TriggerDrive.h
 *========================================================================================================
 *
 * File Description:
 *
 * This implements the IDrive interface to drive the robot using the triggers and bumpers to drive in a tank
 * mode.
 *
 * The left trigger will control the left side of the robot and the right trigger will
 * control the right side of the robot.  Pressing the bumper above the trigger will negate the input.
 * Turning is achieve by making the sides of the robot go in opposite directions (e.g. right side
 * goes forward and the left side in reverse).
 *========================================================================================================
 *  Modifications:
 *  Date            Name            Description of Change
 *  18-Oct-2016     Joe Witcpalek   Created File
 *========================================================================================================*/
#ifndef SRC_TriggerDrive_H_
#define SRC_TriggerDrive_H_


//  Team 302 includes
#include <IDrive.h>             // Interface being implemented
#include <IChassis.h>           // Class that controls the motors on the drive chassis
#include <DragonStick.h>        // Driver Game pad
#include <OperatorInterface.h>  // DragonStick Mapping to the functions


class TriggerDrive: public IDrive
{
    public:
        //--------------------------------------------------------------------
        // Method:      TriggerDrive <<Constructor>>
        // Description: This method is called to create/initialize this class
        //--------------------------------------------------------------------
        TriggerDrive();                    // constructor

        //--------------------------------------------------------------------
        // Method:      Init
        // Description: This method will initialize the values.
        // Returns:     void
        //--------------------------------------------------------------------
        void Init() override;

        //--------------------------------------------------------------------
        // Method:      HasTarget
        // Description: This indicates if this drive mode has a target
        //              such as time, distance, etc.
        // Returns:     true  = has a target
        //              false = doesn't have a target
        //--------------------------------------------------------------------
        bool HasTarget() const override;


        //--------------------------------------------------------------------
        // Method:      SetTarget
        // Description: This sets the target for the drive class.
        // Returns:     void
        //--------------------------------------------------------------------
        void SetTarget
        (
            float   target
        ) override;

        //--------------------------------------------------------------------
        // Method:      DrivePeriodic
        // Description: This method will be called during the periodic loop
        //              to drive the robot.  It will interact with the joysticks
        //              to drive in tank drive mode.
        // Returns:     false = this mode is never done
        //--------------------------------------------------------------------
        bool DrivePeriodic() const override;

        //--------------------------------------------------------------------
        // Method:      GetIdentifier
        // Description: This gets the name that identifies the drive mode..
        // Returns:     const char*         identifier for the drive
        //--------------------------------------------------------------------
        const char* GetIdentifier() const override;

        //--------------------------------------------------------------------
        // Method:      ~TriggerDrive <<destructor>>
        // Description: This method is called to delete these objects
        //--------------------------------------------------------------------
        virtual ~TriggerDrive() = default;

    private:
        // Attributes
        OperatorInterface*      m_oi;
        IChassis*               m_chassis;

        // Default methods we don't want the compiler to implement for us
        TriggerDrive( const TriggerDrive& ) = delete;
        TriggerDrive& operator= ( const TriggerDrive& ) = delete;
        explicit TriggerDrive( TriggerDrive* other ) = delete;

};

#endif /* SRC_TriggerDrive_H_ */
